@@ -60,11 +60,27 @@ namespace Tomboy
 		static string GetDisplayName (Note note)
 		{
 			string display_name = note.Title;
-			if (note.IsNew)
-				display_name += Catalog.GetString (" (new)");
+            int max_length = 40;
+
+			if (note.IsNew) {
+                string new_string = Catalog.GetString(" (new)");
+                max_length -= new_string.Length;
+				display_name = Ellipsify (display_name, max_length)
+                    + new_string;
+            } else {
+                display_name = Ellipsify (display_name, max_length);
+            }
+
 			return FormatForLabel (display_name);
 		}
 
+        static string Ellipsify (string str, int max)
+        {
+            if(str.Length > max)
+                return str.Substring(0, max - 1) + "...";
+            return str;
+        }
+
 		protected override void OnActivated ()
 		{
 			if (!inhibit_activate) {
@@ -506,17 +522,18 @@ namespace Tomboy
 			// it's a 1x1 pixel.  Prevent against this by returning a
 			// reasonable default.  Setting the icon causes OnSizeAllocated
 			// to be called again anyhow.
-			if (panel_size < 16)
-				panel_size = 16;
+			int icon_size = panel_size;
+			if (icon_size < 16)
+				icon_size = 16;
+
 
 			// Control specifically which icon is used at the smaller sizes
 			// so that no scaling occurs.  In the case of the panel applet,
 			// add a couple extra pixels of padding so it matches the behavior
 			// of the notification area tray icon.  See bug #403500 for more
 			// info.
-			int icon_size = panel_size;
 			if (Tomboy.IsPanelApplet)
-				icon_size = panel_size - 2; // padding
+				icon_size = icon_size - 2; // padding
 			if (icon_size <= 21)
 				icon_size = 16;
 			else if (icon_size <= 31)
