@@ -18,6 +18,14 @@ namespace Tomboy
 		                             out int  y,
 		                             out bool push_in)
 		{
+			if (menu.AttachWidget == null) {
+				// Prevent null exception in weird cases
+				x = 0;
+				y = 0;
+				push_in = true;
+				return;
+			}
+			
 			menu.AttachWidget.GdkWindow.GetOrigin (out x, out y);
 			x += menu.AttachWidget.Allocation.X;
 
@@ -226,6 +234,9 @@ namespace Tomboy
 	public class HIGMessageDialog : Gtk.Dialog
 	{
 		Gtk.AccelGroup accel_group;
+		Gtk.VBox extra_widget_vbox;
+		Gtk.Widget extra_widget;
+		Gtk.Image image;
 
 		public HIGMessageDialog (Gtk.Window parent,
 		                         Gtk.DialogFlags flags,
@@ -251,8 +262,6 @@ namespace Tomboy
 			hbox.Show ();
 			VBox.PackStart (hbox, false, false, 0);
 
-			Gtk.Image image = null;
-
 			switch (type) {
 			case Gtk.MessageType.Error:
 				image = new Gtk.Image (Gtk.Stock.DialogError,
@@ -270,11 +279,16 @@ namespace Tomboy
 				image = new Gtk.Image (Gtk.Stock.DialogWarning,
 				                       Gtk.IconSize.Dialog);
 				break;
+			default:
+				image = new Gtk.Image ();
+				break;
 			}
 
-			image.Show ();
-			image.Yalign = 0;
-			hbox.PackStart (image, false, false, 0);
+			if (image != null) {
+				image.Show ();
+				image.Yalign = 0;
+				hbox.PackStart (image, false, false, 0);
+			}
 
 			Gtk.VBox label_vbox = new Gtk.VBox (false, 0);
 			label_vbox.Show ();
@@ -288,6 +302,7 @@ namespace Tomboy
 
 			label = new Gtk.Label (title);
 			label.UseMarkup = true;
+			label.UseUnderline = false;
 			label.Justify = Gtk.Justification.Left;
 			label.LineWrap = true;
 			label.SetAlignment (0.0f, 0.5f);
@@ -296,11 +311,16 @@ namespace Tomboy
 
 			label = new Gtk.Label (msg);
 			label.UseMarkup = true;
+			label.UseUnderline = false;
 			label.Justify = Gtk.Justification.Left;
 			label.LineWrap = true;
 			label.SetAlignment (0.0f, 0.5f);
 			label.Show ();
 			label_vbox.PackStart (label, false, false, 0);
+			
+			extra_widget_vbox = new Gtk.VBox (false, 0);
+			extra_widget_vbox.Show();
+			label_vbox.PackStart (extra_widget_vbox, true, true, 12);
 
 			switch (buttons) {
 			case Gtk.ButtonsType.None:
@@ -334,10 +354,32 @@ namespace Tomboy
 				DestroyWithParent = true;
 		}
 
-		void AddButton (string stock_id, Gtk.ResponseType response, bool is_default)
+		protected void AddButton (string stock_id, Gtk.ResponseType response, bool is_default)
 		{
 			Gtk.Button button = new Gtk.Button (stock_id);
 			button.CanDefault = true;
+			
+			AddButton (button, response, is_default);
+		}
+		
+		protected void AddButton (Gdk.Pixbuf pixbuf, string label_text, Gtk.ResponseType response, bool is_default)
+		{
+			Gtk.Button button = new Gtk.Button ();
+			Gtk.Image image = new Gtk.Image (pixbuf);
+			// NOTE: This property is new to GTK+ 2.10, but we don't
+			//       really need the line because we're just setting
+			//       it to the default value anyway.
+			//button.ImagePosition = Gtk.PositionType.Left;
+			button.Image = image;
+			button.Label = label_text;
+			button.UseUnderline = true;
+			button.CanDefault = true;
+			
+			AddButton (button, response, is_default);
+		}
+		
+		private void AddButton (Gtk.Button button, Gtk.ResponseType response, bool is_default)
+		{
 			button.Show ();
 
 			AddActionWidget (button, response);
@@ -351,6 +393,32 @@ namespace Tomboy
 				                       Gtk.AccelFlags.Visible);
 			}
 		}
+		
+		public Gtk.Widget ExtraWidget
+		{
+			get {
+				return extra_widget;
+			}
+			set {
+				if (extra_widget != null) {
+					extra_widget_vbox.Remove (extra_widget);
+				}
+				
+				extra_widget = value;
+				extra_widget.ShowAll ();
+				extra_widget_vbox.PackStart (extra_widget, true, true, 0);
+			}
+		}
+		
+		/// <value>
+		/// This allows you to set the Gdk.Pixbuf for the dialog's Gtk.Image.
+		/// </value>
+		public Gdk.Pixbuf Pixbuf
+		{
+			set {
+				image.Pixbuf = value;
+			}
+		}
 	}
 
 	public class UriList : ArrayList
@@ -710,22 +778,16 @@ namespace Tomboy
 		}
 	}
 
-	class ToolMenuButton : Gtk.ToggleButton
+	class ToolMenuButton : Gtk.ToggleToolButton
 	{
-		bool is_important;
 		Gtk.Menu menu;
 		Gtk.Image image;
-		Gtk.Label label_horiz;
-		Gtk.Label label_vert;
-		Gtk.Arrow arrow;
-		Gtk.VBox box_vert;
-		Gtk.HBox box_horiz;
 
 		public ToolMenuButton (Gtk.Toolbar toolbar,
 		                       string stock_image,
 		                       string label,
 		                       Gtk.Menu menu)
-: this (toolbar,
+			: this (toolbar,
 		        new Gtk.Image (stock_image, toolbar.IconSize),
 		        label,
 		        menu)
@@ -735,50 +797,31 @@ namespace Tomboy
 		public ToolMenuButton (Gtk.Toolbar toolbar,
 		                       Gtk.Image image,
 		                       string label,
-		                       Gtk.Menu menu)
+		                       Gtk.Menu menu) : base ()
 		{
+			this.IconWidget = image;
+			Gtk.Label l = new Gtk.Label (label);
+			l.UseUnderline = true;
+			this.LabelWidget = l;
 			this.CanFocus = true;
-			this.Relief = toolbar.ReliefStyle;
-			this.FocusOnClick = false;
-
-			this.is_important = false;
-
+//			this.FocusOnClick = false; // TODO: Not supported anymore?
 			this.menu = menu;
-			this.menu.AttachToWidget (this, GuiUtils.DetachMenu);
-			this.menu.Deactivated += ReleaseButton;
+			menu.AttachToWidget (this,GuiUtils.DetachMenu);
+			menu.Deactivated += ReleaseButton;
 
-			this.image = image;
-			label_horiz = new Gtk.Label (label);
-			label_vert = new Gtk.Label (label);
-			arrow = new Gtk.Arrow (Gtk.ArrowType.Down, Gtk.ShadowType.In);
-
-			box_vert = new Gtk.VBox (false, 0);
-			box_vert.PackStart (image, true, true, 0);
-			box_vert.PackStart (label_vert, false, true, 0);
-
-			box_horiz = new Gtk.HBox (false, 0);
-			box_horiz.PackStart (box_vert, true, true, 0);
-			box_horiz.PackEnd (arrow, false, true, 0);
-
-			this.Add (box_horiz);
 			this.ShowAll ();
-			ShowForToolbarStyle (toolbar.ToolbarStyle);
-
-			toolbar.StyleChanged += OnToolbarStyleChanged;
 		}
 
 		protected override bool OnButtonPressEvent (Gdk.EventButton ev)
 		{
 			GuiUtils.PopupMenu (menu, ev);
-			Active = true;
 			return true;
 		}
 
-		protected override void OnActivated ()
+		protected override void OnClicked ()
 		{
 			menu.SelectFirst (true);
 			GuiUtils.PopupMenu (menu, null);
-			Active = true;
 		}
 
 		protected override bool OnMnemonicActivated (bool group_cycling)
@@ -799,53 +842,6 @@ namespace Tomboy
 		{
 			// Release the state when the menu closes
 			Active = false;
-			Release ();
-		}
-
-		public bool IsImportant
-		{
-			get {
-				return is_important;
-			}
-			set {
-				if (value && !is_important) {
-					box_horiz.PackStart (label_horiz, true, true, 2);
-				} else if (!value && is_important) {
-					box_horiz.Remove (label_horiz);
-				}
-				is_important = value;
-			}
-		}
-
-		void ShowForToolbarStyle (Gtk.ToolbarStyle style)
-		{
-			switch (style) {
-			case Gtk.ToolbarStyle.Icons:
-				label_horiz.Hide ();
-				label_vert.Hide ();
-				image.Show ();
-				break;
-			case Gtk.ToolbarStyle.Text:
-				label_vert.Hide ();
-				image.Hide ();
-				label_horiz.Show ();
-				break;
-			case Gtk.ToolbarStyle.Both:
-				label_horiz.Hide ();
-				image.Show ();
-				label_vert.Show ();
-				break;
-			case Gtk.ToolbarStyle.BothHoriz:
-				label_vert.Hide ();
-				image.Show ();
-				label_horiz.Show ();
-				break;
-			}
-		}
-
-		void OnToolbarStyleChanged (object sender, Gtk.StyleChangedArgs args)
-		{
-			ShowForToolbarStyle (args.Style);
 		}
 	}
 
