@@ -86,6 +86,9 @@ namespace Tomboy
 
 			// Avoid lingering note-title after a multi-line insert...
 			Buffer.RemoveTag (title_tag, TitleEnd, end);
+			
+			//In the case of large copy and paste operations, show the end of the block
+			this.Window.Editor.ScrollMarkOnscreen (this.Buffer.InsertMark);
 		}
 
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
@@ -272,7 +275,6 @@ namespace Tomboy
 
 		public override void OnNoteOpened ()
 		{
-			Buffer.TagApplied += TagApplied;
 			Preferences.SettingChanged += OnEnableSpellcheckChanged;
 
 			if ((bool) Preferences.Get (Preferences.ENABLE_SPELLCHECKING)) {
@@ -297,10 +299,14 @@ namespace Tomboy
 				                               null,
 				                               IntPtr.Zero);
 			}
+
+			Buffer.TagApplied += TagApplied;
 		}
 
 		void Detach ()
 		{
+			Buffer.TagApplied -= TagApplied;
+
 			if (obj_ptr != IntPtr.Zero) {
 				gtkspell_detach (obj_ptr);
 				obj_ptr = IntPtr.Zero;
@@ -343,6 +349,23 @@ namespace Tomboy
 			}
 		}
 	}
+	#else
+	// Add in a "dummy" NoteSpellChecker class so that Mono.Addins doesn't
+	// complain at startup.  NoteSpellChecker is specified in Tomboy.addin.xml.
+	public class NoteSpellChecker : NoteAddin
+	{
+		public override void Initialize ()
+		{
+		}
+		
+		public override void Shutdown ()
+		{
+		}
+		
+		public override void OnNoteOpened ()
+		{
+		}
+	}
 	#endif
 
 	public class NoteUrlWatcher : NoteAddin
@@ -936,6 +959,7 @@ namespace Tomboy
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
 		{
 			ApplyWikiwordToBlock (args.Start, args.End);
+			
 		}
 
 		void OnInsertText (object sender, Gtk.InsertTextArgs args)
@@ -944,6 +968,7 @@ namespace Tomboy
 			start.BackwardChars (args.Length);
 
 			ApplyWikiwordToBlock (start, args.Pos);
+			
 		}
 	}
 
